<?php

namespace ByDedi\AdminBundle\Subscriber;

use Doctrine\Common\EventArgs;
use Doctrine\Common\EventSubscriber;
use ByDedi\AdminBundle\Annotation\UploadableAnnotationReader;
use ByDedi\AdminBundle\Helper\UploadHelper;
use Doctrine\Common\Persistence\Mapping\ClassMetadata;
use Doctrine\Common\Persistence\ObjectManager;
use Doctrine\ORM\Event\OnFlushEventArgs;
use Doctrine\ORM\Event\PreFlushEventArgs;

class UploadableSubscriber implements EventSubscriber
{

    /**
     * @var UploadableAnnotationReader
     */
    private $reader;

    /**
     * @var UploadHelper
     */
    private $handler;

    private $filesInfo;

    /**
     * UploadableSubscriber constructor.
     * @param UploadableAnnotationReader $reader
     * @param UploadHelper $handler
     * @param ObjectManager $objectManager
     */
    public function __construct(UploadableAnnotationReader $reader, UploadHelper $handler)
    {
        $this->reader = $reader;
        $this->handler = $handler;
        $this->filesInfo = [];
    }

    /**
     * @return array
     */
    public function getSubscribedEvents()
    {
        return [
            'prePersist',
            'preUpdate',
            'postLoad',
            'postRemove',
            'preFlush',
        ];
    }

    /**
     * @param EventArgs $event
     */
    public function prePersist(EventArgs $event)
    {
        $this->preEvent($event);
    }

    /**
     * @param EventArgs $event
     */
    public function preUpdate(EventArgs $event)
    {
        $this->preEvent($event);
    }

    /**
     * @param EventArgs $event
     */
    public function preFlush(PreFlushEventArgs $event)
    {
        $em = $event->getEntityManager();
        $uow = $em->getUnitOfWork();

        if (!empty($this->filesInfo)) {
            foreach ($this->filesInfo as $info) {
                $entity = $info['entity'];
                if ($uow->isInIdentityMap($entity)) {
                    $targetProperty = $info['annotation']->getFilename();
                    $meta = $em->getClassMetadata(get_class($entity));
                    $newPath = $this->getPropertyValueFromObject($meta, $info['property'], $entity);
                    if ($targetProperty && null !== $newPath) {
                        $path = $this->getPropertyValueFromObject($meta, $targetProperty, $entity);
                        $uow->propertyChanged($entity, $targetProperty, $path, $path);
                        $uow->scheduleForUpdate($entity);
                    }
                }
            }
        }
    }

    /**
     * @param EventArgs $event
     */
    private function preEvent(EventArgs $event)
    {
        $entity = $event->getEntity();
        foreach ($this->reader->getUploadableFields($entity) as $property => $annotation) {
            $this->handler->uploadFile($entity, $property, $annotation);
        }
    }

    /**
     * @param EventArgs $event
     */
    public function postLoad(EventArgs $event)
    {
        $entity = $event->getEntity();
        foreach ($this->reader->getUploadableFields($entity) as $property => $annotation) {
            $this->handler->setFileFromFilename($entity, $property, $annotation);
            $this->filesInfo[] = [
                'entity' => $entity,
                'property' => $property,
                'annotation' => $annotation
            ];
        }
    }

    /**
     * @param EventArgs $event
     */
    public function postRemove(EventArgs $event)
    {
        $entity = $event->getEntity();
        foreach ($this->reader->getUploadableFields($entity) as $property => $annotation) {
            $this->handler->removeFile($entity, $property);
        }
    }


    /**
     * @param ClassMetadata $meta
     * @param $propertyName
     * @param $object
     * @return mixed
     */
    protected function getPropertyValueFromObject(ClassMetadata $meta, $propertyName, $object)
    {
        $refl = $meta->getReflectionClass();
        $filePathField = $refl->getProperty($propertyName);
        $filePathField->setAccessible(true);
        $filePath = $filePathField->getValue($object);

        return $filePath;
    }
}
